@@ -1,11 +1,9 @@
-from __future__ import annotations
-
-import _io
 import multiprocessing
 from dataclasses import dataclass
 from enum import Enum
+from io import TextIOWrapper, StringIO
 from queue import Empty
-from typing import Any, Union
+from typing import Union, List, Dict
 
 s_line_template = 'S\t{low_scan}\t{high_scan}\t{mz}\n'
 i_line_template = 'I\t{keyword}\t{value}\n'
@@ -39,10 +37,10 @@ class Ms2Spectra:
     mz: float
     mass: float
     charge: int
-    info: dict
-    mz_spectra: list[float]
-    intensity_spectra: list[float]
-    charge_spectra: list[float]
+    info: Dict[str, str]
+    mz_spectra: List[float]
+    intensity_spectra: List[float]
+    charge_spectra: List[int]
 
     @property
     def parent_id(self) -> Union[int, None]:
@@ -50,15 +48,17 @@ class Ms2Spectra:
         return int(parent_id) if parent_id else None
 
     @parent_id.setter
-    def parent_id(self, parent_id: Union[str, int]): self.info[ILineKeywords.PARENT_ID_KEYWORD.value] = parent_id
+    def parent_id(self, parent_id: Union[str, int]):
+        self.info[ILineKeywords.PARENT_ID_KEYWORD.value] = parent_id
 
     @property
     def precursor_id(self) -> Union[int, None]:
-        precursor_id =  self.info.get(ILineKeywords.PRECURSOR_ID_KEYWORD.value)
+        precursor_id = self.info.get(ILineKeywords.PRECURSOR_ID_KEYWORD.value)
         return int(precursor_id) if precursor_id else None
 
     @precursor_id.setter
-    def precursor_id(self, precursor_id: Union[str, int]): self.info[ILineKeywords.PRECURSOR_ID_KEYWORD.value] = precursor_id
+    def precursor_id(self, precursor_id: Union[str, int]):
+        self.info[ILineKeywords.PRECURSOR_ID_KEYWORD.value] = precursor_id
 
     @property
     def ook0(self) -> Union[float, None]:
@@ -66,7 +66,8 @@ class Ms2Spectra:
         return float(ook0) if ook0 else None
 
     @ook0.setter
-    def ook0(self, ook0: Union[str, float]): self.info[ILineKeywords.OOK0_KEYWORD.value] = ook0
+    def ook0(self, ook0: Union[str, float]):
+        self.info[ILineKeywords.OOK0_KEYWORD.value] = ook0
 
     @property
     def ccs(self) -> Union[float, None]:
@@ -74,7 +75,8 @@ class Ms2Spectra:
         return float(ccs) if ccs else None
 
     @ccs.setter
-    def ccs(self, ccs: Union[str, float]): self.info[ILineKeywords.CCS_KEYWORD.value] = ccs
+    def ccs(self, ccs: Union[str, float]):
+        self.info[ILineKeywords.CCS_KEYWORD.value] = ccs
 
     @property
     def rt(self) -> Union[float, None]:
@@ -82,15 +84,17 @@ class Ms2Spectra:
         return float(rt) if rt else None
 
     @rt.setter
-    def rt(self, rt: Union[str, float]): self.info[ILineKeywords.RETENTION_TIME_KEYWORD.value] = rt
+    def rt(self, rt: Union[str, float]):
+        self.info[ILineKeywords.RETENTION_TIME_KEYWORD.value] = rt
 
     @property
     def ce(self) -> Union[float, None]:
-        ce =  self.info.get(ILineKeywords.COLLISION_ENERGY_KEYWORD.value)
+        ce = self.info.get(ILineKeywords.COLLISION_ENERGY_KEYWORD.value)
         return float(ce) if ce else None
 
     @ce.setter
-    def ce(self, ce: Union[str, float]): self.info[ILineKeywords.COLLISION_ENERGY_KEYWORD.value] = ce
+    def ce(self, ce: Union[str, float]):
+        self.info[ILineKeywords.COLLISION_ENERGY_KEYWORD.value] = ce
 
     @property
     def iso_mz(self) -> Union[float, None]:
@@ -98,7 +102,8 @@ class Ms2Spectra:
         return float(iso_mz) if iso_mz else None
 
     @iso_mz.setter
-    def iso_mz(self, iso_mz: Union[str, float]): self.info[ILineKeywords.ISOLATION_MZ_KEYWORD.value] = iso_mz
+    def iso_mz(self, iso_mz: Union[str, float]):
+        self.info[ILineKeywords.ISOLATION_MZ_KEYWORD.value] = iso_mz
 
     @property
     def iso_width(self) -> Union[float, None]:
@@ -106,7 +111,8 @@ class Ms2Spectra:
         return float(iso_width) if iso_width else None
 
     @iso_width.setter
-    def iso_width(self, iso_width: Union[str, float]): self.info[ILineKeywords.ISOLATION_WIDTH_KEYWORD.value] = iso_width
+    def iso_width(self, iso_width: Union[str, float]): self.info[
+        ILineKeywords.ISOLATION_WIDTH_KEYWORD.value] = iso_width
 
     @property
     def scan_begin(self) -> Union[int, None]:
@@ -114,7 +120,8 @@ class Ms2Spectra:
         return int(scan_begin) if scan_begin else None
 
     @scan_begin.setter
-    def scan_begin(self, scan_begin: Union[str, int]): self.info[ILineKeywords.SCAN_NUMBER_BEGIN_KEYWORD.value] = scan_begin
+    def scan_begin(self, scan_begin: Union[str, int]):
+        self.info[ILineKeywords.SCAN_NUMBER_BEGIN_KEYWORD.value] = scan_begin
 
     @property
     def scan_end(self) -> Union[int, None]:
@@ -122,7 +129,8 @@ class Ms2Spectra:
         return int(scan_end) if scan_end else None
 
     @scan_end.setter
-    def scan_end(self, scan_end: Union[str, int]): self.info[ILineKeywords.SCAN_NUMBER_END_KEYWORD.value] = scan_end
+    def scan_end(self, scan_end: Union[str, int]):
+        self.info[ILineKeywords.SCAN_NUMBER_END_KEYWORD.value] = scan_end
 
     @property
     def prec_intensity(self) -> Union[float, None]:
@@ -130,7 +138,8 @@ class Ms2Spectra:
         return float(prec_intensity) if prec_intensity else None
 
     @prec_intensity.setter
-    def prec_intensity(self, prec_intensity: Union[str, float]): self.info[ILineKeywords.PRECURSOR_INTENSITY_KEYWORD.value] = prec_intensity
+    def prec_intensity(self, prec_intensity: Union[str, float]):
+        self.info[ILineKeywords.PRECURSOR_INTENSITY_KEYWORD.value] = prec_intensity
 
     def serialize(self) -> str:
         return _serialize_ms2_spectra(self)
@@ -142,21 +151,22 @@ class Ms2Spectra:
 
 def _serialize_ms2_spectra(ms2_spectra: Ms2Spectra) -> str:
     P = 4
-    s_line = s_line_template.format(low_scan=ms2_spectra.low_scan, high_scan=ms2_spectra.high_scan, mz=round(ms2_spectra.mz, P))
+    s_line = s_line_template.format(low_scan=ms2_spectra.low_scan, high_scan=ms2_spectra.high_scan,
+                                    mz=round(ms2_spectra.mz, P))
     i_lines = [i_line_template.format(keyword=k, value=v) for k, v in ms2_spectra.info.items()]
     z_line = z_line_template.format(charge=ms2_spectra.charge, mass=round(ms2_spectra.mass, P))
 
     if ms2_spectra.charge_spectra:
         peak_lines = [peak_line_charged_template.format(mz=round(m, P), intensity=round(i, 1), charge=c) for m, i, c in
-                     zip(ms2_spectra.mz_spectra, ms2_spectra.intensity_spectra, ms2_spectra.charge_spectra)]
+                      zip(ms2_spectra.mz_spectra, ms2_spectra.intensity_spectra, ms2_spectra.charge_spectra)]
     else:
         peak_lines = [peak_line_template.format(mz=round(m, P), intensity=round(i, 1)) for m, i in
-                     zip(ms2_spectra.mz_spectra, ms2_spectra.intensity_spectra)]
+                      zip(ms2_spectra.mz_spectra, ms2_spectra.intensity_spectra)]
 
     return ''.join([s_line] + i_lines + [z_line] + peak_lines)
 
 
-def _deserialize_ms2_spectra(spectra_str: str | list[str], include_spectra=True) -> Ms2Spectra:
+def _deserialize_ms2_spectra(spectra_str: Union[str, List[str]], include_spectra=True) -> Ms2Spectra:
     if isinstance(spectra_str, str):
         lines = spectra_str.split('\n')
     elif isinstance(spectra_str, list):
@@ -198,7 +208,7 @@ def _deserialize_ms2_spectra(spectra_str: str | list[str], include_spectra=True)
                       charge_spectra=charge_spectra)
 
 
-def ms2_spectra_consumer(queue, return_dict):
+def ms2_spectra_consumer(queue: multiprocessing.Queue, return_dict: Dict):
     print('Consumer: Running', flush=True)
     # consume work
     while True:
@@ -212,13 +222,13 @@ def ms2_spectra_consumer(queue, return_dict):
     print('Consumer: Stopping')
 
 
-def from_ms2(ms2_input: str | _io.TextIOWrapper, include_spectra=True, processes=1) -> (list[str], list[Ms2Spectra]):
+# TODO: Remove ms2 queue?
+def from_ms2(ms2_input: Union[str, TextIOWrapper, StringIO], include_spectra=True, processes=1) -> (
+List[str], List[Ms2Spectra]):
     if type(ms2_input) is str:
         lines = ms2_input.split('\n')
-
-    elif type(ms2_input) is _io.TextIOWrapper:
+    elif type(ms2_input) is TextIOWrapper or type(ms2_input) is StringIO:
         lines = ms2_input
-
     else:
         raise ValueError(f'Unsupported input type: {type(ms2_input)}!')
 
@@ -270,7 +280,7 @@ def from_ms2(ms2_input: str | _io.TextIOWrapper, include_spectra=True, processes
     return header_lines, spectra
 
 
-def to_ms2(h_lines: list[str], ms2_spectras: list[Ms2Spectra]) -> str:
+def to_ms2(h_lines: List[str], ms2_spectras: List[Ms2Spectra]) -> str:
     lines = []
     for h_line in h_lines:
         if h_line.endswith('\n'):
